@@ -4,7 +4,10 @@
 
 #include <config.h>
 #include <panel-applet.h>
-#include <egg-screen-help.h>
+#include <libgnomeui/gnome-help.h>
+#ifdef HAVE_GUCHARMAP
+#	include <gucharmap/gucharmap.h>
+#endif
 #include "charpick.h"
 
 
@@ -387,6 +390,30 @@ chooser_button_clicked (GtkButton *button, charpick_data *curr_data)
 	}				    
 }
 
+/* Force the button not to have any focus padding and let the focus
+   indication be drawn on the label itself when space is tight. Taken from the clock applet.
+   FIXME : This is an Evil Hack and should be fixed when the focus padding can be overridden at the gtk+ level */
+
+static inline void force_no_focus_padding (GtkWidget *widget)
+{
+  gboolean first_time=TRUE;
+
+  if (first_time) {
+    gtk_rc_parse_string ("\n"
+			 "   style \"charpick-applet-button-style\"\n"
+			 "   {\n"
+			 "      GtkWidget::focus-line-width=0\n"
+			 "      GtkWidget::focus-padding=0\n"
+			 "   }\n"
+			 "\n"
+			 "    widget \"*.charpick-applet-button\" style \"charpick-applet-button-style\"\n"
+			 "\n");
+    first_time = FALSE;
+  }
+
+  gtk_widget_set_name (widget, "charpick-applet-button");
+}
+
 /* creates table of buttons, sets up their callbacks, and packs the table in
    the event box */
 
@@ -403,6 +430,7 @@ build_table(charpick_data *p_curr_data)
   gint width, height;
   gint max_width=1, max_height=1;
   gint size_ratio;
+
   
   if (p_curr_data->box)
     gtk_widget_destroy(p_curr_data->box);
@@ -434,6 +462,8 @@ build_table(charpick_data *p_curr_data)
   }
   gtk_container_add (GTK_CONTAINER (button), arrow);
   gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
+  /* FIXME : evil hack (see force_no_focus_padding) */
+  force_no_focus_padding (button);
   gtk_box_pack_start (GTK_BOX (box), button, TRUE, TRUE, 0);
   g_signal_connect (G_OBJECT (button), "clicked",
                             G_CALLBACK (chooser_button_clicked), p_curr_data);
@@ -445,15 +475,32 @@ build_table(charpick_data *p_curr_data)
     gchar label[7];
     gint num;
     GtkRequisition req;
+    gchar atk_desc[50];
+    gchar *name;
     
     g_utf8_strncpy (label, charlist, 1);
     charlist = g_utf8_next_char (charlist);
+
+#ifdef HAVE_GUCHARMAP
+    /* TRANSLATOR: This sentance reads something like 'Insert "PILCROW SIGN"'
+     *             hopefully, the name of the unicode character has already
+     *             been translated.
+     */
+    name = g_strdup_printf (_("Insert \"%s\""),
+		    gucharmap_get_unicode_name (g_utf8_get_char (label)));
+#else
+    name = g_strdup (_("Insert special character"));
+#endif
    
     toggle_button[i] = gtk_toggle_button_new_with_label (label);
+    sprintf(atk_desc, _("insert special character %s"), label);
+    set_atk_name_description (toggle_button[i], NULL, atk_desc);
     gtk_widget_show (toggle_button[i]);
     gtk_button_set_relief(GTK_BUTTON(toggle_button[i]), GTK_RELIEF_NONE);
-    gtk_tooltips_set_tip (tooltips, toggle_button[i], 
-                      _("Insert special characters"), NULL);
+    /* FIXME : evil hack (see force_no_focus_padding) */
+    force_no_focus_padding (toggle_button[i]);
+    gtk_tooltips_set_tip (tooltips, toggle_button[i], name, NULL);
+    g_free (name);
                       
     gtk_widget_size_request (toggle_button[i], &req);
     
@@ -462,25 +509,21 @@ build_table(charpick_data *p_curr_data)
   
     g_object_set_data (G_OBJECT (toggle_button[i]), "unichar", 
 				GINT_TO_POINTER(g_utf8_get_char (label)));
-    gtk_signal_connect (GTK_OBJECT (toggle_button[i]), "toggled",
+    g_signal_connect (GTK_OBJECT (toggle_button[i]), "toggled",
                         (GtkSignalFunc) toggle_button_toggled_cb,
                         p_curr_data);
-    gtk_signal_connect (GTK_OBJECT (toggle_button[i]), "button_press_event", 
+    g_signal_connect (GTK_OBJECT (toggle_button[i]), "button_press_event", 
                         (GtkSignalFunc) button_press_hack, p_curr_data->applet);
   }
   
-  switch (panel_applet_get_orient (PANEL_APPLET (p_curr_data->applet))) {
-     	case PANEL_APPLET_ORIENT_DOWN:
-     	case PANEL_APPLET_ORIENT_UP:
-        	size_ratio = p_curr_data->panel_size / max_height;
-        	button_box = gtk_vbox_new (TRUE, 0);
-        	break;
-     	case PANEL_APPLET_ORIENT_LEFT:
-     	case PANEL_APPLET_ORIENT_RIGHT:
-     		size_ratio = p_curr_data->panel_size / max_width;
-     		button_box = gtk_hbox_new (TRUE, 0);
-		break;
+  if (p_curr_data->panel_vertical) {
+    size_ratio = p_curr_data->panel_size / max_width;
+    button_box = gtk_hbox_new (TRUE, 0);
+  } else {
+    size_ratio = p_curr_data->panel_size / max_height;
+    button_box = gtk_vbox_new (TRUE, 0);
   }
+
   gtk_box_pack_start (GTK_BOX (box), button_box, TRUE, TRUE, 0);
   
   size_ratio = MAX (size_ratio, 1);
@@ -492,10 +535,10 @@ build_table(charpick_data *p_curr_data)
   }
   
   for (i = 0; i <len; i++) {  	
-  	float delta = len/(float)size_ratio;
+  	int delta = len/size_ratio;
   	int index;
   	
-  	index = (int) floor (i / delta);  	
+  	index = i / delta;
 	index = CLAMP (index, 0, size_ratio-1);	
   	gtk_box_pack_start (GTK_BOX (row_box[index]), toggle_button[i], TRUE, TRUE, 0);
   }
@@ -508,10 +551,18 @@ build_table(charpick_data *p_curr_data)
   
 }
 
-static void applet_change_pixel_size(PanelApplet *applet, gint size, gpointer data)
+static void applet_size_allocate(PanelApplet *applet, GtkAllocation *allocation, gpointer data)
 {
   charpick_data *curr_data = data;
-  curr_data->panel_size = size;
+  if (curr_data->panel_vertical) {
+    if (curr_data->panel_size == allocation->width)
+      return;
+    curr_data->panel_size = allocation->width;
+  } else {
+    if (curr_data->panel_size == allocation->height)
+      return;
+    curr_data->panel_size = allocation->height;
+  }
 
   build_table (curr_data);
   return;
@@ -563,16 +614,9 @@ about (BonoboUIComponent *uic,
 	return;
   }
   
-  file = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP,
-				    "charpick.png", FALSE, NULL);
-  pixbuf = gdk_pixbuf_new_from_file (file, &error);
-  g_free (file);
+  pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
+		  "charpick", 48, 0, NULL);
    
-  if (error) {
-	g_warning (G_STRLOC ": cannot open %s: %s", file, error->message);
-	g_error_free (error);
-  }
-  
   curr_data->about_dialog = gnome_about_new (_("Character Palette"),
 					     VERSION,
 					     _("Copyright (C) 1998"),
@@ -594,7 +638,7 @@ about (BonoboUIComponent *uic,
   gnome_window_icon_set_from_file (GTK_WINDOW (curr_data->about_dialog),
 				   GNOME_ICONDIR"/charpick.png");
 
-  gtk_signal_connect(GTK_OBJECT(curr_data->about_dialog), "destroy",
+  g_signal_connect(GTK_OBJECT(curr_data->about_dialog), "destroy",
 		     GTK_SIGNAL_FUNC(gtk_widget_destroyed),
 		     &curr_data->about_dialog);
   gtk_widget_show(curr_data->about_dialog);
@@ -609,7 +653,7 @@ help_cb (BonoboUIComponent *uic,
 {
   GError *error = NULL;
 
-  egg_help_display_on_screen (
+  gnome_help_display_on_screen (
 		"char-palette", NULL,
 		gtk_widget_get_screen (curr_data->applet),
 		&error);
@@ -633,7 +677,10 @@ applet_destroy (GtkWidget *widget, gpointer data)
     gtk_widget_destroy (curr_data->about_dialog);   
   if (curr_data->propwindow)
     gtk_widget_destroy (curr_data->propwindow);
-
+  if (curr_data->box)
+    gtk_widget_destroy (curr_data->box);
+  if (curr_data->menu)
+    gtk_widget_destroy (curr_data->menu);
   g_free (curr_data);
   
 }
@@ -736,7 +783,7 @@ charpicker_applet_fill (PanelApplet *applet)
   GList *list;
   gchar *string;
   
-  gnome_window_icon_set_default_from_file (GNOME_ICONDIR"/charpick.png");
+  gtk_window_set_default_icon_name ("charpick");
   
   panel_applet_add_preferences (applet, "/schemas/apps/charpick/prefs", NULL);
   panel_applet_set_flags (applet, PANEL_APPLET_EXPAND_MINOR);
@@ -787,10 +834,10 @@ charpicker_applet_fill (PanelApplet *applet)
 			    GDK_SELECTION_CLIPBOARD,
                             utf8_atom,
 			    0);
-  gtk_signal_connect (GTK_OBJECT (curr_data->applet), "selection_get",
+  g_signal_connect (GTK_OBJECT (curr_data->applet), "selection_get",
 		      GTK_SIGNAL_FUNC (charpick_selection_handler),
 		      curr_data);
-  gtk_signal_connect (GTK_OBJECT (curr_data->applet), "selection_clear_event",
+  g_signal_connect (GTK_OBJECT (curr_data->applet), "selection_clear_event",
 		      GTK_SIGNAL_FUNC (selection_clear_cb),
 		      curr_data);
  
@@ -800,8 +847,8 @@ charpicker_applet_fill (PanelApplet *applet)
   g_signal_connect (G_OBJECT (applet), "change_orient",
 		    G_CALLBACK (applet_change_orient), curr_data);
 
-  g_signal_connect (G_OBJECT (applet), "change_size",
-		    G_CALLBACK (applet_change_pixel_size), curr_data);
+  g_signal_connect (G_OBJECT (applet), "size_allocate",
+		    G_CALLBACK (applet_size_allocate), curr_data);
 		    
   g_signal_connect (G_OBJECT (applet), "destroy",
   		    G_CALLBACK (applet_destroy), curr_data);
@@ -826,7 +873,7 @@ charpicker_applet_fill (PanelApplet *applet)
 					NULL);
   }
 
-			             
+  register_stock_for_edit ();			             
   populate_menu (curr_data);
   
   return TRUE;
